{-----------------------------------------------------------------------------
 Unit Name: Intro
 Author:    Nitrogen
 Purpose:   Introduction to the demo..
 History:   04 July '02 - first creation.
-----------------------------------------------------------------------------}
unit Intro;

interface

Uses OpenGL12, Globals;

Procedure intro_init;
Procedure intro_loop;
Procedure intro_unload;

Var FinishedIntro: boolean = false;

implementation

uses Log, LookupTable, TGATexture, ParticleText;

type
  Vertex = array[0..3] of Glfloat;
  Vertex3 = array[0..2] of Glfloat;
  Vertex2 = Array[0..1] of Glfloat;

  TGLCoord = Record
    X, Y, Z : glFLoat;
  end;
  TMetaBall = Record
    Radius : glFloat;
    X, Y, Z : glFLoat;
  end;
  TGridPoint = record
    Pos : TGLCoord;
    Normal : TGLCoord;
    Value : glFLoat;  // Result of the metaball equations at this point
  end;
  PGridPoint = ^TGridPoint;
  TGridCube = record
    GridPoint : Array [0..7] of PGridPoint; // Points to 8 grid points (cube)
  end;

Const
  GridSize = 30;
  MAX_TITLE = 100;
  Rad: array[1..4] of single = (0.2, 0.2, 0.2, 0.2);
  Piby180 = Pi / 180;
  By6 = 1/600;
  By5 = 1/500;
  By4 = 1/400;
  By3 = 1/300;


var
  MetaBall : Array[1..4] of TMetaBall;
  Grid  : Array[0..40, 0..40, 0..40] of TGridPoint;
  Col   : Array[0..40, 0..40, 0..40] of Vertex;
  Cubes : Array[0..30, 0..30, 0..30] of TGridCube;
  TessTriangles : Integer;           // Number of triangles by metaball tesselation.

  Title, TTitle: array[0..MAX_TITLE, 0..MAX_TITLE] of Vertex;
  TitleTex, TTitleTex: array[0..MAX_TITLE, 0..MAX_TITLE] of Vertex2;

  R,G,B: array[1..4] of glFloat;
  TR,TG,TB: array[1..4] of glFloat;
  L: array[0..3] of Vertex;

  //For Title sCreen
  VX, Xx: glFloat;
  Phase2: boolean = false;


  EnvTex, LogoTex: glUint;

  Text: array[0..5] of PTextObj;


procedure NormalizeVector(var V : TGLCoord);
var Length : glFloat;
begin
  Length :=Sqrt(V.x*V.x + V.y*V.y + V.z*V.z);
  if Length = 0 then exit;

  V.x :=V.x / Length;
  V.y :=V.y / Length;
  V.z :=V.z / Length;
end;

procedure InitGrid;
var cx, cy, cz : Integer;
begin
  // Create the grid positions
  for cx := 0 to GridSize do
    for cy := 0 to GridSize do
      for cz := 0 to GridSize do
      begin
        Grid[cx, cy, cz].Pos.X := 2*cx/GridSize -1;   // grid from -1 to 1
        Grid[cx, cy, cz].Pos.Y := 2*cy/GridSize -1;   // grid from -1 to 1
        Grid[cx, cy, cz].Pos.Z := 1-2*cz/GridSize;    // grid from -1 to 1
        Col[Cx, Cy, Cz][0] := 0;
        Col[Cx, Cy, Cz][1] := 0;
        Col[Cx, Cy, Cz][2] := 0;
      end;

  // Create the cubes. Each cube points to 8 grid points
  for cx := 0 to GridSize-1 do
    for cy := 0 to GridSize-1 do
      for cz := 0 to GridSize-1 do
      begin
        Cubes[cx,cy,cz].GridPoint[0] := @Grid[cx,   cy,   cz  ];
        Cubes[cx,cy,cz].GridPoint[1] := @Grid[cx+1, cy,   cz  ];
        Cubes[cx,cy,cz].GridPoint[2] := @Grid[cx+1, cy,   cz+1];
        Cubes[cx,cy,cz].GridPoint[3] := @Grid[cx,   cy,   cz+1];
        Cubes[cx,cy,cz].GridPoint[4] := @Grid[cx,   cy+1, cz  ];
        Cubes[cx,cy,cz].GridPoint[5] := @Grid[cx+1, cy+1, cz  ];
        Cubes[cx,cy,cz].GridPoint[6] := @Grid[cx+1, cy+1, cz+1];
        Cubes[cx,cy,cz].GridPoint[7] := @Grid[cx,   cy+1, cz+1];
      end;
end;

{----------------------------------------------------------}
{  Interpolate the position where an metaballs intersects  }
{  the line betweenthe two coordicates, C1 and C2          }
{----------------------------------------------------------}
procedure Interpolate(const C1, C2 : TGridPoint; var CResult, Norm : TGLCoord);
var mu : glFLoat;
begin
  if Abs(C1.Value) = 1 then
  begin
    CResult := C1.Pos;
    Norm := C1.Normal;
  end
  else
  if Abs(C2.Value) = 1 then
  begin
    CResult := C2.Pos;
    Norm := C2.Normal;
  end
  else
  if C1.Value = C2.Value then
  begin
    CResult := C1.Pos;
    Norm := C1.Normal;
  end
  else
  begin
    mu := (1 - C1.Value) / (C2.Value - C1.Value);
    CResult.x := C1.Pos.x + mu * (C2.Pos.x - C1.Pos.x);
    CResult.y := C1.Pos.y + mu * (C2.Pos.y - C1.Pos.y);
    CResult.z := C1.Pos.z + mu * (C2.Pos.z - C1.Pos.z);

    Norm.X := C1.Normal.X + (C2.Normal.X - C1.Normal.X) * mu;
    Norm.Y := C1.Normal.Y + (C2.Normal.Y - C1.Normal.Y) * mu;
    Norm.Z := C1.Normal.Z + (C2.Normal.Z - C1.Normal.Z) * mu;
  end;
end;


{------------------------------------------------------------}
{  Calculate the triangles required to draw a Cube.          }
{  Draws the triangles that makes up a Cube                  }
{------------------------------------------------------------}
procedure CreateCubeTriangles(const GridCube : TGridCube);//; const X,Y,Z: integer);
var I : Integer;
    CubeIndex: Integer;
    VertList, Norm : Array[0..11] of TGLCoord;
begin
  // Determine the index into the edge table which tells
  // us which vertices are inside/outside the metaballs
  CubeIndex := 0;
  if GridCube.GridPoint[0]^.Value < 1 then CubeIndex := CubeIndex or 1;
  if GridCube.GridPoint[1]^.Value < 1 then CubeIndex := CubeIndex or 2;
  if GridCube.GridPoint[2]^.Value < 1 then CubeIndex := CubeIndex or 4;
  if GridCube.GridPoint[3]^.Value < 1 then CubeIndex := CubeIndex or 8;
  if GridCube.GridPoint[4]^.Value < 1 then CubeIndex := CubeIndex or 16;
  if GridCube.GridPoint[5]^.Value < 1 then CubeIndex := CubeIndex or 32;
  if GridCube.GridPoint[6]^.Value < 1 then CubeIndex := CubeIndex or 64;
  if GridCube.GridPoint[7]^.Value < 1 then CubeIndex := CubeIndex or 128;

  // Check if the cube is entirely in/out of the surface
  if edgeTable[CubeIndex] = 0 then
    Exit;

  // Find the vertices where the surface intersects the cube.
  with GridCube do
  begin
    if (edgeTable[CubeIndex] and 1) <> 0 then
      Interpolate(GridPoint[0]^, GridPoint[1]^, VertList[0], Norm[0]);
    if (edgeTable[CubeIndex] and 2) <> 0 then
      Interpolate(GridPoint[1]^, GridPoint[2]^, VertList[1], Norm[1]);
    if (edgeTable[CubeIndex] and 4) <> 0 then
      Interpolate(GridPoint[2]^, GridPoint[3]^, VertList[2], Norm[2]);
    if (edgeTable[CubeIndex] and 8) <> 0 then
      Interpolate(GridPoint[3]^, GridPoint[0]^, VertList[3], Norm[3]);
    if (edgeTable[CubeIndex] and 16) <> 0 then
      Interpolate(GridPoint[4]^, GridPoint[5]^, VertList[4], Norm[4]);
    if (edgeTable[CubeIndex] and 32) <> 0 then
      Interpolate(GridPoint[5]^, GridPoint[6]^, VertList[5], Norm[5]);
    if (edgeTable[CubeIndex] and 64) <> 0 then
      Interpolate(GridPoint[6]^, GridPoint[7]^, VertList[6], Norm[6]);
    if (edgeTable[CubeIndex] and 128) <> 0 then
      Interpolate(GridPoint[7]^, GridPoint[4]^, VertList[7], Norm[7]);
    if (edgeTable[CubeIndex] and 256) <> 0 then
      Interpolate(GridPoint[0]^, GridPoint[4]^, VertList[8], Norm[8]);
    if (edgeTable[CubeIndex] and 512) <> 0 then
      Interpolate(GridPoint[1]^, GridPoint[5]^, VertList[9], Norm[9]);
    if (edgeTable[CubeIndex] and 1024) <> 0 then
      Interpolate(GridPoint[2]^, GridPoint[6]^, VertList[10], Norm[10]);
    if (edgeTable[CubeIndex] and 2048) <> 0 then
      Interpolate(GridPoint[3]^, GridPoint[7]^, VertList[11], Norm[11]);
  end;

  I := 0;
  while TriangleTable[CubeIndex, i] <> -1 do
  begin
    //glColor3fv(@Col[X,Y,Z]);
    glNormal3fv(@Norm[TriangleTable[CubeIndex, i]]);
    glVertex3fv(@VertList[TriangleTable[CubeIndex][i]]);

    glNormal3fv(@Norm[TriangleTable[CubeIndex, i+1]]);
    glVertex3fv(@VertList[TriangleTable[CubeIndex][i+1]]);

    glNormal3fv(@Norm[TriangleTable[CubeIndex, i+2]]);
    glVertex3fv(@VertList[TriangleTable[CubeIndex][i+2]]);

    Inc(TessTriangles);
    Inc(i, 3);
  end;
end;


{------------------------------------------------------------------}
{  Function to draw the actual scene                               }
{------------------------------------------------------------------}
procedure drawMetaballs;
var cx, cy, cz : Integer;
    I, J: Integer;
    Xx, Yy, Zz, T: glFloat;
begin


  glTranslatef(0.0,0.0,-1.7);

  // calculate metaball positions
  if (Elapsedtime <= 48077) then
  begin
   MetaBall[1].X :=-0.3*cos(elapsedTime*by6) - 0.01*cos(elapsedTime*by6);
   MetaBall[1].Y :=0.4*sin(elapsedTime*by5) + 0.02*cos(elapsedTime*by6);
   MetaBall[1].Z :=0.2*sin(elapsedTime*by5) + 0.02*cos(elapsedTime*by6);

   MetaBall[2].X :=0.6*sin(elapsedTime*by4) - 0.05*cos(elapsedTime*by6);
   MetaBall[2].Y :=0.4*sin(elapsedTime*by5) + 0.02*cos(elapsedTime*by6);
   MetaBall[2].Z :=-0.3*cos(elapsedTime*by5) + 0.03*cos(elapsedTime*by6);

   MetaBall[3].X :=0.5*sin(elapsedTime*by4) - 0.05*cos(elapsedTime*by6);
   MetaBall[3].Y :=0.5*sin(elapsedTime*by6) + 0.05*cos(elapsedTime*by6);
   MetaBall[3].Z :=-0.4*cos(elapsedTime*by3) + 0.02*cos(elapsedTime*by6);

   MetaBall[4].X :=0.6*sin(elapsedTime*by3) - 0.02*cos(elapsedTime*by6);
   MetaBall[4].Y :=0.2*sin(elapsedTime*by6) + 0.03*cos(elapsedTime*by4);
   MetaBall[4].Z :=-0.4*cos(elapsedTime*by3) + 0.05*cos(elapsedTime*by5);
  end
  else
  begin
    Metaball[1].X := -0.42;
    Metaball[1].Y := 0;
    Metaball[1].Z := 0;
    Metaball[2].X := 0.42;
    Metaball[2].Y := 0;
    Metaball[2].Z := 0;
    Metaball[3].X := 0;
    Metaball[3].Y := -0.42;
    Metaball[3].Z := 0;
    Metaball[4].X := 0;
    Metaball[4].Y := 0.42;
    Metaball[4].Z := 0;
  end;

  if (Elapsedtime >= 45077) and (Elapsedtime <= 48077) then
  begin
    T := (Elapsedtime-45077)/(3000);
    Metaball[1].X := Metaball[1].X*(1-T)-0.42*(T);
    Metaball[1].Y := Metaball[1].Y*(1-T)-0*(T);
    Metaball[1].Z := Metaball[1].Z*(1-T)-0*(T);
    Metaball[2].X := Metaball[2].X*(1-T)+0.42*(T);
    Metaball[2].Y := Metaball[2].Y*(1-T)+0*(T);
    Metaball[2].Z := Metaball[2].Z*(1-T)+0*(T);
    Metaball[3].X := Metaball[3].X*(1-T)-0*(T);
    Metaball[3].Y := Metaball[3].Y*(1-T)-0.42*(T);
    Metaball[3].Z := Metaball[3].Z*(1-T)-0*(T);
    Metaball[4].X := Metaball[4].X*(1-T)+0*(T);
    Metaball[4].Y := Metaball[4].Y*(1-T)+0.42*(T);
    Metaball[4].Z := Metaball[4].Z*(1-T)+0*(T);
  end;



  TessTriangles := 0;
  For cx := 0 to GridSize do
    For cy := 0 to GridSize do
      For cz := 0 to GridSize do
        with Grid[cx, cy, cz] do
        begin
         Value :=0;
        Col[Cx, Cy, Cz][0] := 0;
        Col[Cx, Cy, Cz][1] := 0;
        Col[Cx, Cy, Cz][2] := 0;

          for I :=1 to 4 do  // go through all meta balls
          begin
            with Metaball[I] do
               begin
                 XX := Pos.x-x;
                 Yy := Pos.y-y;
                 Zz := Pos.z-z;
                 Value := Value + Radius*Radius/(xx*xx + yy*yy + zz*zz);
                 //Value := Value-(Radius-YY);
                 {if I = 1 then
                 begin
                 Col[cx, Cy, Cz][0] := Col[cx, Cy, Cz][0] + Radius*Radius/(xx*xx + yy*yy + zz*zz)
                 end
                 else if I = 2 then
                 begin
                 Col[cx, Cy, Cz][1] := Col[cx, Cy, Cz][1] + Radius*Radius/(xx*xx + yy*yy + zz*zz)
                 end
                 else if I = 3 then
                 begin
                 Col[cx, Cy, Cz][2] := Col[cx, Cy, Cz][2] + Radius*Radius/(xx*xx + yy*yy + zz*zz)
                 end
                 else
                 begin
                 Col[cx, Cy, Cz][2] := Col[cx, Cy, Cz][2] + Radius*Radius/(xx*xx + yy*yy + zz*zz);
                 Col[cx, Cy, Cz][1] := Col[cx, Cy, Cz][1]  + Radius*Radius/(xx*xx + yy*yy + zz*zz);
                 Col[cx, Cy, Cz][0] := Col[cx, Cy, Cz][0] + Radius*Radius/(xx*xx + yy*yy + zz*zz);

                 end;
                 }
               end;
          end;
        end;

  // Calculate normals at the grid vertices
  For cx := 1 to GridSize-1 do
  begin
    For cy := 1 to GridSize-1 do
    begin
      For cz := 1 to GridSize-1 do
      begin
        Grid[cx,cy,cz].Normal.X := Grid[cx-1, cy, cz].Value - Grid[cx+1, cy, cz].Value;
        Grid[cx,cy,cz].Normal.Y := Grid[cx, cy-1, cz].Value - Grid[cx, cy+1, cz].Value;
        Grid[cx,cy,cz].Normal.Z := Grid[cx, cy, cz-1].Value - Grid[cx, cy, cz+1].Value;
      end;
    end;
  end;

{
  // Draw the metaballs by drawing the triangle in each cube in the grid
 glPushMatrix();

   glRotatef(Sin(elapsedTime/6000)*50,0, 0, 1);
    glBegin(GL_TRIANGLES);
      For cx := 0 to GridSize-1 do
        for cy := 0 to GridSize-1 do
          for cz := 0 to GridSize-1 do
            CreateCubeTriangles(Cubes[cx, cy, cz]);
    glEnd;
 glPopMatrix();
 }

  //Draw Transparent version over the original.

  glenable(GL_BLEND);
  glDisable(GL_DEPTH_TEST);
  glPushMatrix();
    glRotatef(Sin(elapsedTime/6000)*50,0, 0, 1);
    glScalef(1.1, 1.1, 1.1);
    glBegin(GL_TRIANGLES);
      For cx := 0 to GridSize-1 do
        for cy := 0 to GridSize-1 do
          for cz := 0 to GridSize-1 do
            CreateCubeTriangles(Cubes[cx, cy, cz]);
    glEnd;
  glPopMatrix();
  glDisable(GL_BLEND);
  glEnable(GL_DEPTH_TEST);


end;


procedure InitMetaballs;
begin
  // initialise the metaball size and positions
 begin
  MetaBall[1].Radius :=Rad[1];
  MetaBall[1].X :=0;
  MetaBall[1].Y :=0;
  MetaBall[1].Z :=0;

  MetaBall[2].Radius :=Rad[2];
  MetaBall[2].X :=0;
  MetaBall[2].Y :=0;
  MetaBall[2].Z :=0;

  MetaBall[3].Radius :=rad[3];
  MetaBall[3].X :=0;
  MetaBall[3].Y :=0;
  MetaBall[3].Z :=0;

  MetaBall[4].Radius :=rad[4];
  MetaBall[4].X :=0;
  MetaBall[4].Y :=0;
  MetaBall[4].Z :=0;


  InitGrid;
 end;

    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glEnable(GL_DEPTH_TEST);
    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
    glTexGeni(GL_S, GL_SPHERE_MAP, 0);
    glTexGeni(GL_T, GL_SPHERE_MAP, 0);
    glDisable(GL_BLEND);
    glEnable(GL_TEXTURE_GEN_S);
    glEnable(GL_TEXTURE_GEN_T);

end;

Function Make(X, Y, Z, W: glFloat): Vertex;
begin
  Result[0] := X;
  Result[1] := Y;
  Result[2] := Z;
  Result[3] := W;
end;

Function Make3(X, Y, Z: glFloat): Vertex3;
begin
  Result[0] := X;
  Result[1] := Y;
  Result[2] := Z;
end;

procedure SetLights;
Var C: Vertex;
    I: integer;
    SpotDir: Vertex3;
    Spot: glFloat;
begin


 for I := 0 to 3 do
 begin
  glLightfv(gl_light0+I, GL_POSITION, @L[I]);

 if ElapsedTime <= 50077 then
 begin

  if (ElapsedTime <= 1000) and (Text[0].State = 0) then
  begin
   Text[0].Start(elapsedTime);
  end;
  if ((ElapsedTime >= 14100) and (ElapsedTime <= 14300)) and (Text[1].State = 0) then
  Text[1].Start(elapsedTime);
  if ((ElapsedTime >= 22052) and (ElapsedTime <= 22252)) and (Text[2].State = 0) then
  Text[2].Start(elapsedTime);
  if ((ElapsedTime >= 29900) and (ElapsedTime <= 31000)) and (Text[3].State = 0) then
  Text[3].Start(elapsedTime);
  if ((ElapsedTime >= 37900) and (ElapsedTime <= 38000)) and (Text[4].State = 0) then
  Text[4].Start(elapsedTime);


  if ((ElapsedTime > 0) and (ElapsedTime <= 14200)) or (ElapsedTime > 38000) then
  begin
    if R[1] < Tr[1] then R[1] := R[1] + 0.001;
    if G[1] < TG[1] then G[1] := G[1] + 0.001;
    if B[1] < TB[1] then B[1] := B[1] + 0.001;
  end
  else
  begin
    if R[1] > 0 then R[1] := R[1] - 0.001;
    if G[1] > 0 then G[1] := G[1] - 0.001;
    if B[1] > 0 then B[1] := B[1] - 0.001;
  end;

  if ((ElapsedTime > 14200) and (ElapsedTime <= 22152)) or (ElapsedTime > 38000) then
  begin
    if R[2] < Tr[2] then R[2] := R[2] + 0.001;
    if G[2] < TG[2] then G[2] := G[2] + 0.001;
    if B[2] < TB[2] then B[2] := B[2] + 0.001;
  end
  else
  begin
    if R[2] > 0 then R[2] := R[2] - 0.001;
    if G[2] > 0 then G[2] := G[2] - 0.001;
    if B[2] > 0 then B[2] := B[2] - 0.001;
  end;
  if ((ElapsedTime > 22152) and (ElapsedTime <= 30000)) or (ElapsedTime > 38000) then
  begin
    if R[3] < Tr[3] then R[3] := R[3] + 0.001;
    if G[3] < TG[3] then G[3] := G[3] + 0.001;
    if B[3] < TB[3] then B[3] := B[3] + 0.001;
  end
  else
  begin
    if R[3] > 0 then R[3] := R[3] - 0.001;
    if G[3] > 0 then G[3] := G[3] - 0.001;
    if B[3] > 0 then B[3] := B[3] - 0.001;
  end;
  if (ElapsedTime > 30000) then
  begin
    if R[4] < Tr[4] then R[4] := R[4] + 0.001;
    if G[4] < TG[4] then G[4] := G[4] + 0.001;
    if B[4] < TB[4] then B[4] := B[4] + 0.001;
  end
  else
  begin
    if R[4] > 0 then R[4] := R[4] - 0.001;
    if G[4] > 0 then G[4] := G[4] - 0.001;
    if B[4] > 0 then B[4] := B[4] - 0.001;
  end;

 end
 else
 begin
  if R[I+1] > 0 then R[I+1] := R[I+1] - 0.005;
  if G[I+1] > 0 then G[I+1] := G[I+1] - 0.005;
  if B[I+1] > 0 then B[I+1] := B[I+1] - 0.005;
 end;
  C := Make(R[I+1]  , G[I+1] ,B[I+1]  , 1); //Light colour
  glLightfv(gl_light0+I, GL_DIFFUSE, @C);

 end;

  Spot := 45;
  glLightfv(gl_light0, GL_SPOT_CUTOFF, @Spot);
  glLightfv(gl_light1, GL_SPOT_CUTOFF, @Spot);
  glLightfv(gl_light2, GL_SPOT_CUTOFF, @Spot);
  glLightfv(gl_light3, GL_SPOT_CUTOFF, @Spot);

  Spotdir := Make3(-1,-1,1);
  glLightfv(gl_light0, GL_SPOT_DIRECTION, @SpotDir);
  Spotdir := Make3( 1,-1,1);
  glLightfv(gl_light1, GL_SPOT_DIRECTION, @SpotDir);
  Spotdir := Make3( 1,1,1);
  glLightfv(gl_light2, GL_SPOT_DIRECTION, @SpotDir);
  Spotdir := Make3(-1, 1,1);
  glLightfv(gl_light3, GL_SPOT_DIRECTION, @SpotDir);



end;

Procedure InitTitle;
Var X, Y, XX, YY: integer;
    D: glFloat;
begin
if Not Phase2  then
   For Y := 0 to MAX_TITLE do
   begin
   For X := 0 to MAX_TITLE do
    begin
       TTitleTex[X,Y][0] := X/MAX_TITLE;
       TTitleTex[X,Y][1] := Y/MAX_TITLE;

       XX := X-(MAX_TITLE div 2);
       YY := Y-(MAX_TITLE div 2);

       D := Sqrt(XX*XX+YY*YY);


       TitleTex[X,Y][0] := Sin(D/15+(X+Y)/150)*1;
       TitleTex[X,Y][1] := Cos(D/15+(X-Y)/150)*1;

       TTitle[X,Y][0] := ((X/MAX_TITLE)-0.5)*10;
       TTitle[X,Y][1] := ((Y/MAX_TITLE)-0.5)*10;
       TTitle[X,Y][2] := 0;

       Title[X,Y][0] := Sin(D/30-(X+Y)/200)*2.1;
       Title[X,Y][1] := Cos(D/30+(X-Y)/200)*2.1;
       Title[X,Y][2] := Sin(D/30-(X+Y)/200)*2;



    end;
   end
else
   For Y := 0 to MAX_TITLE do
   begin
   For X := 0 to MAX_TITLE do
    begin

       XX := X-(MAX_TITLE div 2);
       YY := Y-(MAX_TITLE div 2);

       D := Sqrt(XX*XX+YY*YY);


       TTitleTex[X,Y][0] := (X/MAX_TITLE)-Sin(D/20)*0.4;
       TTitleTex[X,Y][1] := (Y/MAX_TITLE)-Cos(D/20)*0.4;

       TTitle[X,Y][0] := ((X/MAX_TITLE)-0.5)*40+Sin(D/50)*4;
       TTitle[X,Y][1] := ((Y/MAX_TITLE)-0.5)*40+Cos(D/50)*4;
       TTitle[X,Y][2] := 0;

    end;
   end;

{  For X := 1 to MAX_TITLE-1 do
  For Y := 1 to MAX_TITLE-1 do
  begin
     Title[X,Y][0] := (Title[X,Y][0]+Title[X+1,Y][0]+Title[X-1,Y][0]+Title[X,Y+1][0]+Title[X,Y-1][0])/5;
     Title[X,Y][1] := (Title[X,Y][1]+Title[X+1,Y][1]+Title[X-1,Y][1]+Title[X,Y+1][1]+Title[X,Y-1][1])/5;
     Title[X,Y][2] := (Title[X,Y][2]+Title[X+1,Y][2]+Title[X-1,Y][2]+Title[X,Y+1][2]+Title[X,Y-1][2])/5;


  end;
 }

end;


Procedure intro_Init;
var I: integer;
begin
   Addtolog('intro INIT');
   LoadJPG('Data\Env.jpg', EnvTex);
   LoadJPG('Data\LogoTex.jpg', LogoTex);
   InitParticleText;

   InitTitle;

   Text[0].Load('Data\Air.ppl');
   Text[1].Load('Data\Fire.ppl');
   Text[2].Load('Data\Earth.ppl');
   Text[3].Load('Data\Water.ppl');
   Text[4].Load('Data\T4e.ppl');
  // Text[5].Load('Data\Logo.ppl');


   Text[0].Z := -100;
   Text[0].X :=  50;
   Text[0].Y :=  50;

   Text[4].Z := -70;
   Text[4].X :=  -40;
   Text[4].Y :=  25;


   Text[1].Z := -100;
   Text[1].X :=  -90;
   Text[1].Y :=  50;

   Text[2].Z := -100;
   Text[2].X :=  -90;
   Text[2].Y :=  -55;

   Text[3].Z := -100;
   Text[3].X :=  40;
   Text[3].Y :=  -55;

   Text[0].R := 0.9;
   Text[0].G := 0.9;
   Text[0].B := 1;

   Text[4].R := 1;
   Text[4].G := 1;
   Text[4].B := 1;


   Text[1].R := 1;
   Text[1].G := 0.4;
   Text[1].B := 0.2;


   Text[2].R := 0.2;
   Text[2].G := 0.9;
   Text[2].B := 0.2;

   Text[3].R := 0.2;
   Text[3].G := 0.4;
   Text[3].B := 1;

   Text[0].TimePause := 1000;
   Text[0].Timeout := 5000;

   Text[4].TimeIn := 16000;
   Text[4].EraseMethod := 4;
   Text[4].TimePause := 3000;
   Text[4].Timeout := 8000;

   Text[1].EraseMethod := 1;
   Text[1].TimePause := 1000;
   Text[1].Timeout := 4000;

   Text[2].EraseMethod := 2;
   Text[2].TimePause := 1000;
   Text[2].Timeout := 4000;

   Text[3].EraseMethod := 3;
   Text[3].TimePause := 1000;
   Text[3].Timeout := 5000;

{   Text[5].Z := -40;
   Text[5].X :=  -30;
   Text[5].Y :=  25;
   Text[5].R := 1;
   Text[5].G := 0.98;
   Text[5].B := 0.97;
   Text[5].EraseMethod := 5;
   Text[5].TimePause := 4000;
   Text[5].TimeIn := 17000;
   Text[5].Timeout := 5000;
   For I := 0 to high(Text[5].P) do
   Text[5].P[I].Size := Text[5].P[I].Tc/5;
 }
   InitMetaBalls;

   Tr[1] := 1;
   Tg[1] := 1;
   Tb[1] := 1;

   Tr[2] := 1;
   Tg[2] := 0.3;
   Tb[2] := 0;

   Tr[3] := 0.2;
   Tg[3] := 0.8;
   Tb[3] := 0.2;

   Tr[4] := 0.1;
   Tg[4] := 0.4;
   Tb[4] := 1;

   L[0] := Make(10, 10, -10, 0); //Light position in world
   L[1] := Make(-10, 10, -10, 0); //Light position in world
   L[2] := Make(-10, -10, -10, 0); //Light position in world
   L[3] := Make(10, -10, -10, 0); //Light position in world


  glEnable(GL_LIGHTING);
  //GlEnable(GL_COLOR_MATERIAL);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHT1);
  glEnable(GL_LIGHT2);
  glEnable(GL_LIGHT3);


   SetLights;

end;


Procedure intro_loop;
Var I, X, Y: integer;
    T, T2: glFloat;
begin

if (Elapsedtime <= 55000) then
begin
  glEnable(GL_TEXTURE_GEN_S);
  glEnable(GL_TEXTURE_GEN_T);

  DrawMetaballs;
  setLights;

  glTranslatef(0,0,-10);

  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);
  glenable(GL_BLEND);

  glDisable(GL_TEXTURE_GEN_S);
  glDisable(GL_TEXTURE_GEN_T);

  glBindTexture(GL_TEXTURE_2D, ParticleTex);

  for I := 0 to 4 do
  begin
     glPushMatrix;
      Text[I].Draw;
     glPopMatrix;
  end;

  glenable(GL_LIGHTING);
  glenable(GL_DEPTH_TEST);
  glDisable(GL_BLEND);
  glbindTexture(GL_TEXTURE_2D, EnvTex);
end
else
begin
//  if ((Elapsedtime >= 54500) and (Elapsedtime <= 55500)) and (Text[5].State = 0) then
//  Text[5].Start(elapsedtime);

  glTranslatef(0,0,-9);

  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);
  glEnable(GL_BLEND);
  glDisable(GL_CULL_FACE);

  glBindTexture(GL_TEXTURE_2D, LogoTex);


  if Phase2 then
   T := (ElapsedTime-58458)/(50000)
  else
   T := (ElapsedTime-55000)/(60458-55000);

  For X := 0 to MAX_TITLE do
  For Y := 0 to MAX_TITLE do
  begin

     Title[X,Y][0] := Title[X,Y][0]*(1-T)+ TTitle[X,Y][0]*T;
     Title[X,Y][1] := Title[X,Y][1]*(1-T)+ TTitle[X,Y][1]*T;
     Title[X,Y][2] := Title[X,Y][2]*(1-T)+ TTitle[X,Y][2]*T;


     TitleTex[X,Y][0] := TitleTex[X,Y][0]*(1-T)+ TTitleTex[X,Y][0]*T;
     TitleTex[X,Y][1] := TitleTex[X,Y][1]*(1-T)+ TTitleTex[X,Y][1]*T;



  end;

   if (Phase2 = false) and (ElapsedTime > 58458) then
   begin
    Phase2 := True;
    Vx := 0;
    XX := 0;
    InitTitle;
   end;


   if not Phase2 then
   begin
   Vx := Vx + (ElapsedTime-55000)/100000;
   Xx := xX + Vx*0.2;
   end;

  for I := 0 to 6 do
  if (xx*I) < 9 then
  begin
  if Phase2 then
  begin
    T2 := abs(TTitle[0,0][0]-Title[0,0][0])/2;
    if T2 <= 0.001 then
      begin
         StageChange := true;
         Stage := 2;
      end;
  end
  else
    T2 := 1-((xx*I)/9);
  glColor3f(T2, T2, T2);
  if Phase2 then
    if I > 0 then Continue;
  For Y := 0 to MAX_TITLE-1 do
  begin
  glBegin(GL_TRIANGLE_STRIP);
  For X:= 0 to MAX_TITLE do
   begin

      glTexCoord2fv(@TitleTex[X,Y]);
      glVertex3fv(@Title[X,Y]);
      glTexCoord2fv(@TitleTex[X,Y+1]);
      glVertex3fv(@Title[X,Y+1]);
   end;
  glend;
  end;
  glTranslatef(0,0,xx);
  end;

end;

end;
Procedure intro_unload;
begin
   Addtolog('Intro Unload');

   glDeleteTextures(1, @EnvTex);
   glDeleteTextures(1, @ParticleTex);
   glDeleteTextures(1, @LogoTex);

   Text[0].Free;
   Text[1].Free;
   Text[2].Free;
   Text[3].Free;


end;

end.
