{-----------------------------------------------------------------------------
 Unit Name: ParticleText
 Author:    Nitrogen
 Purpose:   Displays some fancy particle text..
 History:
-----------------------------------------------------------------------------}

unit ParticleText;


interface

Uses OpenGL12;


Var ParticleTex: glUint;

type
    PParticle = ^Particle;
    Particle = record
                X,Y,Z, Tx, Ty, Tz, R, G, B: glFloat;
                Vx, Vy, Vz: glFloat;
                C, Tc: glFloat;
                Size, S: glFloat;
               end;
    PTextObj = object
                X,Y,Z, R, G, B, Ier: glFloat;
                P: array of Particle;

                State: integer;
                TimeIn, TimePause, TimeOut, TimeStart: LongWord;

                EraseMethod, HalfP: integer;
                Procedure Load(F: string);
                Procedure Start(time: longword);
                Procedure Draw;
                Procedure Free;
               end;

Procedure InitParticleText;

implementation

Uses Log, TGATexture, Globals;

Procedure InitParticleText;
begin
    LoadJPG('Data\Particle.jpg', ParticleTex);

end;

Procedure PTextObj.Load(F: string);
Var Ff: file of Byte;
    By: byte;
begin
  Addtolog('Loading ParticleText Obj:' + F);
 State := 0;
 Try
  SetLength(P, 0);
  AssignFile(Ff, F);
  Reset(Ff);
  repeat
     Setlength(P,  High(P)+2);
     FillChar(P[High(P)], Sizeof(Particle), 0);
     Read(Ff, By);
     P[High(P)].tX := By/3;
     Read(Ff, By);
     P[High(P)].tY := -By/3;
     Read(Ff, By);
     P[High(P)].Tc := By/256;
     P[High(P)].Size := 0.5;
  until EOF(Ff);
  CloseFile(Ff);

 except
 Addtolog('Error in ParticleText obj, Attempting to close file...');
 CloseFile(ff);
 end;

  Addtolog('ParticleText: ' + inttostr(high(P)) + ' particles loaded.');
  HalfP := High(P) div 2;

end;

Procedure PTextObj.Free;
begin
   Setlength(P, 0);
end;

Procedure PTextObj.Start(time: longword);
Var I: integer;
begin
  TimeStart := Time;
  State := 1;
  Ier := 0;
  if EraseMethod = 0 then
  For I := 0 to high(P) do
  begin
     P[I].X := (random(1000)-500)/10;
     P[I].Y := (random(1000)-500)/10;
     P[I].Z := (random(1000)-500)/10;
   end;
  if EraseMethod = 5 then
  For I := 0 to high(P) do
  begin
     P[I].X := Sin(Pi/180*(360/High(P))*I/2)*Random(200);
     P[I].Y := Cos(Pi/180*(360/High(P))*I/2)*Random(200);
     P[I].Z := (random(1000)-500)/100;
   end;
  if EraseMethod = 4 then
  For I := 0 to high(P) do
  begin
     P[I].X := (random(1000)-500);
     P[I].Y := (random(1000)-500);
     P[I].Z := (random(1000)-500);
   end;
  if EraseMethod = 1 then
  For I := 0 to high(P) do
  begin
     P[I].X := P[I].Tx;
     P[I].Y := (-random(1000))/10;
     P[I].Z := (random(1000)-500)/10;
   end;
  if EraseMethod = 2 then
  For I := 0 to high(P) do
  begin
     P[I].X := P[I].Tx;
     P[I].Y := (random(1000))/10;
     P[I].Z := (random(1000)-500)/10;
   end;
  if EraseMethod = 3 then
  For I := 0 to high(P) do
  begin
     P[I].X := (random(1000)-500)/100;
     P[I].Y := (random(1000)-500)/100;
     P[I].Z := (random(1000)-500)/100;
   end;
   TimeIn := 6000;
end;


Procedure DrawParticle(P: PParticle);
Var S, X, Y, Z: glFloat;
begin
  // glPushMatrix;
  // glTranslatef(P^.X,P^.Y,P^.Z);
   X := P^.X; Y := P^.Y; Z := P^.Z;

   S := P^.S;
   glColor3f(P^.R,P^.G,P^.B);
    glBegin(GL_TRIANGLE_STRIP);
      glTexcoord2f(0,0); glVertex3f(X-S, y+S, z);
      glTexcoord2f(0,1); glVertex3f(X-S, y-S, z);
      glTexcoord2f(1,0); glVertex3f(X+S, y+S, z);
      glTexcoord2f(1,1); glVertex3f(X+S, y-S, z);
    glEnd;

  // glPopMatrix;

end;

Procedure PTextObj.Draw;
Var I: integer;
    Ic, T, XX, YY: single;
begin
  if (State = 0) or (State = 4) then exit;


  glTranslatef(X,Y,Z);
  if state = 1 then
  begin
  Ic := (elapsedTime-TimeStart)/TimeOut;
     For I := 0 to high(P) do
     begin
     with P[I] do
     begin
      XX := X-P[HalfP].X;
      YY := Y-P[HalfP].Y;
      S := Size;//*(1+Sin(elapsedtime/1000+Sqrt(Xx*Xx+Yy*Yy)/2)*0.3);
        if EraseMethod = 4 then
        begin
        X := X + (Tx-X)/(39);
        y := y + (Ty-y)/(39);
        z := z + (Tz-z)/(39);

        C := Tc;
        end
        else if EraseMethod = 5 then
        begin
        X := X + (Tx-X)/(39+I/180);
        y := y + (Ty-y)/(39+I/180);
        z := z + (Tz-z)/(39+I/180);

        If C < Tc then C := C + X/(400-(Ic*200));

//        S := 0.4+Sin(elapsedTime/150+(10-X+10-Y)/100)*0.2;

        end
        else
        begin
        if abs(Tx-X) > 0.02 then
        X := X + (Tx-X)/(45)*1.1;
        if abs(Ty-Y) > 0.02 then
        y := y + (Ty-y)/(45)*1.1;
        if abs(Tz-z) > 0.02 then
        z := z + (Tz-z)/(45)*1.1;
        If C < Tc then C := C + X/(1400-(Ic*200));
        end;

     end;
        P[I].R := R*P[I].C;
        P[I].G := G*P[I].C;
        P[I].B := B*P[I].C;

     end;

//     if elapsedTime-TimeStart >= TimeIn then
//     begin
//     State := 2;
//     end;
     if elapsedTime-TimeStart >= TimeIn+TimePause then
     begin
      State := 3;
     end;
  end;

 { if State = 2 then
  begin
    { If EraseMethod = 5 then
     For I := 1 to high(P) do
     with P[I] do
     begin
        Vx := Vx + (0.5-(Random(1000)/1000))/700;
        Vy := Vy + (0.5-(Random(1000)/1000))/700;
        vZ := vZ + Tc/1000;
        Z := Z + Vz;
        X := X + Vx;
        Y := Y + Vy;
        if Size > 0.3 then Size := Size - 0.001;
     end;

  end; }

  if state = 3 then
  begin
  Ic := (elapsedTime-TimeStart-TimeIn-TimePause)/TimeOut;
  ic := Ic *0.01;
     Ier := Ier + 0.01;
     For I := 0 to high(P) do
     with P[I] do
     begin
      XX := X-P[HalfP].X;
      YY := Y-P[HalFP].Y;
      S := Size;//*(1+Sin(elapsedtime/1000+Sqrt(Xx*Xx+Yy*Yy)/2)*0.3);
      if EraseMethod = 0 then
      if Ier >= (I/High(P)) then
      begin
        Vx := Vx + (0.5-(Random(1000)/1000))/400;
        Vy := Vy + (0.5-(Random(1000)/1000))/400;
        Vz := Vz + (0.5-(Random(1000)/1000))/10000;
        X := X + Vx;
        Y := Y + Vy;
        Z := Z + Vz;
        Size := Size + Ic*4;
        If R > 0 then R := R - Ic;
        If g > 0 then g := g - Ic;
        If b > 0 then b := b - Ic;
      end;
      if EraseMethod = 4 then
      if Ier >= (I/High(P)) then
      begin
        Vx := Vx + Sin(ElapsedTime/(500+I+Y/10))*0.01;//(0.5-(Random(1000)/1000))/200;
        Vy := Vy + Cos(ElapsedTime/(500+I+X/10))*0.01;//(0.5-(Random(1000)/1000))/200;
        Vz := Vz + Cos(ElapsedTime/500+I)*0.01;//(0.5-(Random(1000)/1000))/200;
        X := X + Vx;
        Y := Y + Vy;
        Z := Z + Vz;
        Size := Size + Ic;
        If R > 0 then R := R - Ic;
        If g > 0 then g := g - Ic;
        If b > 0 then b := b - Ic;
      end;
      if EraseMethod = 5 then
      if Ier >= (I/High(P)) then
      begin
        Vx := Vx + (0.5-(Random(1000)/1000))/20;
        Vy := Vy + (0.5-(Random(1000)/1000))/20;
        Vz := Vz + (0.5-(Random(1000)/1000))/20;
        X := X + Vx;
        Y := Y + Vy;
        Z := Z + Vz;
        Size := Size - Ic;
        If R > 0 then R := R - Ic;
        If g > 0 then g := g - Ic;
        If b > 0 then b := b - Ic;
      end;
      if EraseMethod = 1 then
      if Ier >= (I/High(P)) then
      begin
        Vx := Vx + (0.4-(Random(1000)/1000))/200;
        Vy := Vy - ((0.4-(Random(1000)/1000))/400)*((10-tY)/10);
        Vz := Vz + (0.5-(Random(1000)/1000))/10000;
        X := X + Vx;
        Y := Y + Vy;
        Z := Z + Vz;
        Size := Size + Ic;
        If R > 0 then R := R - Ic;
        If g > 0 then g := g - Ic;
        If b > 0 then b := b - Ic;

      end;
      if EraseMethod = 2 then
      if Ier >= (I/High(P)) then
      begin
        Vx := Vx + (0.5-(Random(1000)/1000))/100;
        Vy := Vy + (0.5-(Random(1000)/1000))/100 - 0.0001*X;
        Vz := Vz + (0.5-(Random(1000)/1000))/10000;
        X := X + Vx;
        Y := Y + Vy;
        Z := Z + Vz;
        Size := Size + Ic*3;
        If R > 0 then R := R - Ic*2;
        If g > 0 then g := g - Ic*2;
        If b > 0 then b := b - Ic*2;

      end;
      if EraseMethod = 3 then
      if Ier >= (I/High(P)) then
      begin
        Vx := Vx - (0.5-(Random(1000)/1000))/400;
        Vy := Vy - (0.5-(Random(1000)/1000))/400;
        Vz := Vz - (0.5-(Random(1000)/1000))/400;
        X := X + Vx;
        Y := Y + Vy;
        Z := Z + Vz;
        Size := Size + Ic*3;
        If R > 0 then R := R - Ic;
        If g > 0 then g := g - Ic;
        If b > 0 then b := b - Ic;

      end;

     end;

     if elapsedTime-TimeStart >= TimeIn+TimePause+TimeOut then
     State := 4;

  end;


  For I := 0 to high(P) do
  DrawParticle(@P[I]);


end;

end.
