unit Sound;

interface

Uses FMod;

Var SoundEnabled: boolean = False;

Type MusicStream = Object
                   channel: integer;
                   H: PFsoundStream;
                   Function Load(Filename: string): boolean;
                   Procedure Play;
                   Procedure Stop;
                   Procedure Free;
                   Procedure Pause;
                   end;
    TrackedStream = Object
                     channel: integer;
                     H: PFMusicModule;
                     Function Load(Filename: string): boolean;
                     Function Play: boolean;
                     Procedure Stop;
                     Procedure Free;
                     Procedure SetOrder(ind: integer);
                    end;


Procedure InitSound;
Procedure ShutdownSound;



implementation

Uses Windows, Log;

Procedure InitSound;
begin
  SoundEnabled := False;
  if FMOD_VERSION > FSOUND_GetVersion then
  begin
    Addtolog('FMod Version Mismatch! Sound will be disabled');
    Exit;
  end;

  FSOUND_SetOutput(FSOUND_OUTPUT_DSOUND);
  FSOUND_SetDriver(0);
  FSOUND_SetMixer(FSOUND_MIXER_AUTODETECT);


  if not FSOUND_Init(44100, 16, 0) then
  begin
    Addtolog('FAILED to init Sound. Sound will be disabled');

    Messagebox(0, 'FMod Failed to start.', 'Error!', MB_ICONERROR or MB_OK);
    FSOUND_Close();
  end;

  SoundEnabled := True;
  Addtolog('Sound engine started.');
end;

Procedure ShutdownSound;
begin
  FSOUND_Close();
end;

Function MusicStream.Load(Filename:string): boolean;
begin

 Result := False;
 if SoundEnabled then
 begin
  h := FSOUND_Stream_OpenFile(PChar(filename), FSOUND_LOOP_NORMAL or FSOUND_NORMAL, 0);
  if h <> nil then
   Result := True;
 end;

 if Result then
 Addtolog('Loaded Musicstream ' + filename + ' - Success')
 else
 Addtolog('Loaded Musicstream ' + filename + ' - Failed')

end;

Procedure MusicStream.Play;
begin
 if Soundenabled then
 channel := FSOUND_Stream_Play(FSOUND_FREE, h);
end;


Procedure MusicStream.Stop;
begin
 if Soundenabled then
  FSOUND_Stopsound(channel);
end;

Procedure MusicStream.Pause;
begin
 if Soundenabled then
  FSOUND_SetPaused(channel, not FSOUND_GetPaused(channel))
end;

Procedure MusicStream.Free;
begin
 if Soundenabled then
  FSOUND_Stream_Close(h);
end;


Function TrackedStream.Load(Filename:string): boolean;
begin

 Result := False;
 h := Nil;
 if SoundEnabled then
 begin
  h := FMUSIC_LoadSong(PChar(filename));
  if h <> nil then
      begin
        FMUSIC_SetMasterVolume(h, 255);
        Result := True;
        if (FMUSIC_GetType(h) = FMUSIC_TYPE_MOD) or (FMUSIC_GetType(h) = FMUSIC_TYPE_S3M) then
        begin
         Addtolog('TrackedStream: filetype is MOD or S3M. Setting 15% crossover.');
          FMUSIC_SetPanSeperation(h, 0.15);  // 15% crossover
        end;
      end;


 end;

 if Result then
 Addtolog('Loaded Trackedstream ' + filename + ' - Success')
 else
 Addtolog('Loaded Trackedstream ' + filename + ' - Failed')

end;


function trackedStream.Play: boolean;
begin
 if Soundenabled then
 result := FMUSIC_PlaySong(h)

 else
 Result := false;
end;

Procedure TrackedStream.SetOrder(ind: integer);
begin
    FMUSIC_SetOrder(H,ind);

end;


Procedure trackedStream.Stop;
begin
 if Soundenabled then
  FMUSIC_StopSong(h);
end;

Procedure trackedStream.Free;
begin
 if Soundenabled then
  FMUSIC_FreeSong(h);
end;


end.
